"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var effects_1 = require("@ngrx/effects");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var querystring = require("querystring");
var shared_1 = require("../../../shared");
var models_1 = require("../../../base/models");
var base_component_1 = require("./base.component");
/**
 * Componente base para listar objectos de tipo T utilizando filtros de búsqueda de tipo F mediante el framework ngrx-data
 */
var BaseListComponent = /** @class */ (function (_super) {
    __extends(BaseListComponent, _super);
    function BaseListComponent(actions$, tableViewService, activatedRoute, router, entityService, newFilterFunc) {
        var _this = _super.call(this) || this;
        _this.actions$ = actions$;
        _this.tableViewService = tableViewService;
        _this.activatedRoute = activatedRoute;
        _this.router = router;
        _this.entityService = entityService;
        _this.newFilterFunc = newFilterFunc;
        _this.customToolbarActions = {};
        _this.breadcrumb = _this.createBreadcrumb();
        _this.toolbar$ = _this.createToolbar(false, false);
        return _this;
    }
    BaseListComponent.prototype.ngOnInit = function () {
        console.log("ngOnInit called");
        this.updateToolbar();
        this.updateItems();
        this.updatePageStatus();
        this.updateFilter();
        this.initSearch();
    };
    BaseListComponent.prototype.ngOnDestroy = function () {
        this.destroy$.next(true);
        this.destroy$.unsubscribe();
    };
    /** Primer valor de la toolbar al crear el objeto BaseList */
    BaseListComponent.prototype.createToolbar = function (readOnly, enable) {
        return rxjs_1.of(models_1.makeToolbarList(readOnly, enable));
    };
    /** Decide lanzar la búsqueda al crear el componente o no en función de si en la URL de carga del componente se ha especificado el parámetro
     * query a true
     */
    BaseListComponent.prototype.initSearch = function () {
        var query = this.activatedRoute.snapshot.params["query"];
        console.log("parametro query con valor " + query);
        if (query == "true") {
            this.search(this.filter);
        }
    };
    /**actualización del toolbar en función de si hay elementos cargados tras la búsqueda o no hay elementos*/
    BaseListComponent.prototype.updateToolbar = function () {
        var _this = this;
        this.entityService.entities$.pipe(operators_1.takeUntil(this.destroy$)).subscribe(function (items) {
            _this.toolbar$ = _this.createToolbar(false, items && items.length > 0);
        });
    };
    /**actualización de los items y la cuenta de items*/
    BaseListComponent.prototype.updateItems = function () {
        this.items$ = this.entityService.entities$;
        this.totalItems$ = this.entityService.count$;
    };
    /**actualiza el page status en función de tres observables. El estado de envío al servidor, el posible error producido y el éxito de la operación*/
    BaseListComponent.prototype.updatePageStatus = function () {
        var _this = this;
        this.status$ = rxjs_1.merge(this.entityService.loading$.pipe(operators_1.map(function (loading) {
            if (loading) {
                return shared_1.PageStatus.Searching;
            }
        })), effects_1.createEffect(function () {
            return _this.actions$.pipe(operators_1.map(function (action) { return action.type; }), operators_1.filter(function (actionType) { return actionType.indexOf("error") != -1; }), operators_1.map(function (action) { return shared_1.PageStatus.Error; }));
        }, { dispatch: false }), effects_1.createEffect(function () {
            return _this.actions$.pipe(operators_1.map(function (action) { return action.type; }), operators_1.filter(function (actionType) { return actionType.indexOf("success") != -1; }), operators_1.map(function (action) { return shared_1.PageStatus.FinishSearch; }));
        }, { dispatch: false }));
    };
    /**actualización del filtro de búsqueda*/
    BaseListComponent.prototype.updateFilter = function () {
        var _this = this;
        this.entityService.filter$.pipe(operators_1.takeUntil(this.destroy$)).subscribe(function (filter) {
            if (filter) {
                _this.filter = _this.newFilterFunc(JSON.parse(filter));
                console.log("filter recibido", _this.filter);
            }
        });
    };
    /** el componente toolbar nos comunica que se ha pulsado un botón */
    BaseListComponent.prototype.onToolbarButtonPressed = function (button) {
        console.log("Button pressed:" + button);
        if (this.customToolbarActions[button]) {
            this.customToolbarActions[button]();
        }
        else if (button === shared_1.BitToolbarComponent.PRINT_BUTTON) {
            this.handlePrint();
        }
        else if (button === shared_1.BitToolbarComponent.EXPORT_BUTTON) {
            this.handleExport();
        }
        else if (button === shared_1.BitToolbarComponent.NEW_BUTTON) {
            this.handleNew();
        }
        else {
            this.otherAction(button);
        }
    };
    /** el componente toolbar nos comunica que se ha pulsado un botón */
    BaseListComponent.prototype.handlePrint = function () {
        window.print();
    };
    /** Cuando tenemos que exportar el listado */
    BaseListComponent.prototype.handleExport = function () {
        if (this.tableViewService) {
            this.tableViewService.sendExportMessage(); //enviamos un mensaje a un observable para que el tableview active el export
        }
        else {
            alert("Exportació no implementada per aquest llistat");
        }
    };
    /** Cuando tenemos que nagevar al formulario para crear un nuevo elemento*/
    BaseListComponent.prototype.handleNew = function () {
        this.gotoForm();
    };
    /**cualquier otra accion que no sea la típica de guardar o eliminar */
    BaseListComponent.prototype.otherAction = function (button) {
        console.warn("otherAction " + button + " pressed");
    };
    /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de elementos a mostrar por página */
    BaseListComponent.prototype.onNewPageSize = function (pageSize) {
        console.log("onNewPageSize " + pageSize);
        this.filter.pagina = 1; // reseteamos a la página inicial para evitar comportamienos extraños
        this.filter.resultados = pageSize;
        this.entityService.setFilter(JSON.stringify(this.filter));
        this.search(this.filter);
    };
    /** Se invocará cuando el componente paginator hijo nos comunique que el usuario ha cambiado el número de página a mostrar */
    BaseListComponent.prototype.onPageChange = function (page) {
        console.log("onPageChange " + page);
        this.filter.pagina = page;
        this.entityService.setFilter(JSON.stringify(this.filter));
        this.search(this.filter);
    };
    /** Lanza la búsqueda contra el servidor */
    BaseListComponent.prototype.search = function (filter) {
        console.log("search called ", filter);
        this.entityService.setFilter(JSON.stringify(filter));
        //indicamos al store que la lista no se ha cargado porque la queremos volver a cargar. Esto evita efectos raros
        this.entityService.setLoaded(false);
        this.entityService.getWithQuery(querystring.stringify(filter));
    };
    /** Resetea el filtro de búsqueda */
    BaseListComponent.prototype.reset = function (filter) {
        console.log("reset called ", filter);
        this.entityService.setFilter(JSON.stringify(this.filter));
    };
    return BaseListComponent;
}(base_component_1.BaseComponent));
exports.BaseListComponent = BaseListComponent;
//# sourceMappingURL=base-list.component.js.map